(* Copyright (C) 2000, HELM Team.
 * 
 * This file is part of HELM, an Hypertextual, Electronic
 * Library of Mathematics, developed at the Computer Science
 * Department, University of Bologna, Italy.
 * 
 * HELM is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * HELM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HELM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * For details, see the HELM World-Wide-Web page,
 * http://cs.unibo.it/helm/.
 *)

exception NotWellTyped of string;;

let typecheck uri =
  let rec typecheck_term curi t =
  let module T = Theory in
  let module P = CicTypeChecker in
  let module C = CicCache in
  let module U = UriManager in
  let obj_typecheck uri =
   try
    P.typecheck (U.uri_of_string uri)
   with
    P.NotWellTyped s ->
     raise (NotWellTyped
      ("Type Checking was NOT successfull due to an error during " ^
       "type-checking of term " ^ uri ^ ":\n\n" ^ s))
  in
    match t with
       T.Theorem uri -> obj_typecheck (curi ^ "/" ^ uri)
     | T.Definition uri -> obj_typecheck (curi ^ "/" ^ uri)
     | T.Axiom uri -> obj_typecheck (curi ^ "/" ^ uri)
     | T.Variable uri -> obj_typecheck (curi ^ "/" ^ uri)
     | T.Section (uri,l) -> typecheck_theory l (curi ^ "/" ^ uri)
 and typecheck_theory l curi =
  List.iter (typecheck_term curi) l
 in
  let (uri, l) = TheoryCache.get_theory uri in
   typecheck_theory l uri
;;
